@@ -8,3 +8,6 @@ from .http import HTTPStream
 from .rtmpdump import RTMPStream
 from .streamprocess import StreamProcess
 from .wrappers import StreamIOWrapper, StreamIOThreadWrapper
+
+from .flvconcat import extract_flv_header_tags
+from .playlist import Playlist, FLVPlaylist
